/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import java.rmi.RemoteException;

import org.dom4j.Document;
import org.dom4j.io.DOMReader;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spagosoa.components.jobs.BckDynDataProxy;
import it.eng.spagosoa.components.jobs.ParametersType;
import it.eng.spagosoa.metadb.model.ServiceAssembly;

public class BackupInstancesAction extends AbstractAction {
	private String _processo = null;
	private String _version;
	private String _dataInizio= null;
	private String _dataFine = null;
	
	private boolean _backupData = false;
	private boolean _deleteData = false;
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		BckDynDataProxy proxy = new BckDynDataProxy();
		ParametersType aParametersType = new ParametersType();
		if(_processo != null){
			ServiceAssembly serviceAssembly = ProcessBO.getProcessDetail(Long.parseLong(_processo));
			aParametersType.setServiceAssemblyName(serviceAssembly.getName());
		}
		if(_version != null)
			aParametersType.setServiceAssemblyVersion(_version);
		if(_dataInizio != null)
			 aParametersType.setFromDate(_dataInizio);
		if(_dataFine != null)
			aParametersType.setToDate(_dataFine);
		
		Object response = null;
		Document document = null;
		DOMReader xmlReader = new DOMReader();
		//call web Services to backup 
		if(_backupData) {
			proxy.setEndpoint("http://localhost:9092/bckdyndata/");
			try {
				response = proxy.run(aParametersType);
				document = xmlReader.read((org.w3c.dom.Document)response);
				if (document.asXML().contains("<OK>")) 
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10006", "properties.console.messages"));
				else
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10007", "properties.console.messages"));
			} catch(RemoteException re) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "BackupInstancesAction::service:: backupdata", re);
				getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10004", "properties.console.messages"));
				return;
			}
		}
		if(_deleteData) {
			proxy.setEndpoint("http://localhost:9092/deldyndata/");
			try {
				response = proxy.run(aParametersType);
				document = xmlReader.read((org.w3c.dom.Document)response);
				if (document.asXML().contains("<OK>")) 
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10008", "properties.console.messages"));
				else
					getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10009", "properties.console.messages"));
			} catch(RemoteException re) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "BackupInstancesAction::service:: deletedata", re);
				getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.INFORMATION, "10005", "properties.console.messages"));
			}
		}
		
	}
	
	private void loadParametersFromRequest(SourceBean request) {
		if(request.getAttribute(Constants.LBL_BACKUP) != null)
			_backupData = true;
		if(request.getAttribute(Constants.LBL_DELETE) != null)
			_deleteData = true;
		_processo = (String)request.getAttribute(Constants.LBL_PROCESS_ID);
		_version = (String)request.getAttribute(Constants.LBL_VERSION);
		_dataInizio = (String)request.getAttribute(Constants.LBL_DATA_INIZIO);
		_dataFine = (String) request.getAttribute(Constants.LBL_DATA_FINE);
	}
}
