/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.hibernate.Criteria;
import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.AbstractPage;
import it.eng.console.hibernate.pagination.CriteriaHibernatePage;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.process.action.ProcessListFilteredAction;
import it.eng.console.process.bo.RelevantDataBO;
import it.eng.console.util.Constants;
import it.eng.spagosoa.metadb.model.Attribute;
import it.eng.spagosoa.metadb.model.AttributeIter;
import it.eng.spagosoa.metadb.model.InstanceSa;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

public class ProcessInstanceListFilteredAction extends ProcessListFilteredAction {
	private String _processId = null;
	private String _taxonomyName = null;
	private String _taxonomyValue = null;
	private String _idSelectedNode = null;			
	private String _iterType = null;	

	public static final String CORRELATION_ID_PREFIX = "RESTARTED_ID:";

	public void service(SourceBean serviceRequest, SourceBean serviceResponse) throws Exception {
		//esistono 4 casi in cui viene invocata questa action
		//processId  valorizzato (ma non iterType): cerco tutte le istanze del processo avente id==processId
		//processId e iterType sono valorizzati: cerco tutte le istanze del processo aventi istanza iter di tipo iterType
		//se taxonomyName+taxonomyValue valorizzato (ma non processId): cerco tutte le istanze dei processo categorizzati secondo i valori passati
		//iterType  valorizzato (ma non processId): cerco qualsiasi istanze di processo aventi istanza iter di tipo iterType
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		int page = getRequestPage(serviceRequest, sessionContainer, Constants.PAGE_INSTANCES);
		
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		
		//********* recupero (se ci sono) parametri legati all'iter e all'istanza
		List instanceIterInpuParameters = getInstanceIterInput(serviceRequest);
		
		//save info for pagination
		SourceBean processBeanList = new SourceBean("LIST");
		saveParametersForPagination(processBeanList);
		
		//genero bean contente attributi di istanza di iter
		if(_iterType!=null) {
			SourceBean attributesList = createRelevantIterAttributes(_iterType, serviceRequest);
			serviceResponse.setAttribute(attributesList);
		}
		
		//call ProcessInstanceBO
		List processInstanceList = null;
		Query query = null;
		Criteria criteria = null;
		AbstractPage hibernatePage = null;
		
		if(_processId!=null) {
			if(_iterType==null)
				query = ProcessInstanceBO.getProcessInstanceList((new Long(_processId)).longValue());
			else
				query = ProcessInstanceBO.getProcessInstanceList((new Long(_processId)).longValue(),(new Long(_iterType)).longValue(), instanceIterInpuParameters);
		}else {
			if(_taxonomyName!=null && _taxonomyValue!=null) 
				criteria = ProcessInstanceBO.getProcessInstanceList(_taxonomyName, _taxonomyValue);
			else if(_iterType != null)
				query = ProcessInstanceBO.getProcessInstanceListByIterType((new Long(_iterType)).longValue(), instanceIterInpuParameters);
			else
				query = ProcessInstanceBO.getProcessInstanceList();
		}
		
		//pagination
		hibernatePage = (query != null) ? 
						new QueryHibernatePage(query, page, Constants.sizeProcessInstanceListFiltered)
		 				: new CriteriaHibernatePage(criteria, page, Constants.sizeProcessInstanceListFiltered);
//		if (page > hibernatePage.getLastPageNumber()){
//			hibernatePage.setPage(--page);
//			sessionContainer.setAttribute(Constants.PAGE, page);
//		} else if(page == Constants.LAST_PAGE_VALUE){
//			int lastPage = hibernatePage.getLastPageNumber();
//			hibernatePage.setPage(lastPage);
//			sessionContainer.setAttribute(Constants.PAGE, lastPage);
//		}
		processInstanceList = hibernatePage.getList();
		
		//create response
		Iterator processInstancesIt = processInstanceList.iterator();		
		Object processInstance = null;
		while(processInstancesIt.hasNext()) {
			processInstance = processInstancesIt.next();
			if(processInstance instanceof InstanceSa) 
				processBeanList.setAttribute(createProcessInstanceBean((InstanceSa)processInstance));
			else
				processBeanList.setAttribute(createProcessInstanceBean((List)processInstance));	
		} 
		serviceResponse.setAttribute(processBeanList);
	}
	
	private void loadParametersFromRequest(SourceBean request) {
		_processId = (String)request.getAttribute(Constants.LBL_PROCESS_ID);
		_taxonomyName = (String)request.getAttribute(Constants.LBL_SELECTED_TAXONOMY);
		_taxonomyValue = (String)request.getAttribute(Constants.LBL_SELECTED_TAXONOMY_VALUE);
		_idSelectedNode = (String)request.getAttribute(Constants.LBL_SELECTED_NODE);			
		_iterType = (String)request.getAttribute(Constants.LBL_SELECTED_ITER);	
	}
	
	private void saveParametersForPagination(SourceBean processBeanList) throws SourceBeanException {
		try {
			if(_processId != null)
				processBeanList.setAttribute(Constants.LBL_PROCESS_ID, _processId);
			if(_idSelectedNode!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_NODE, _idSelectedNode);
			if(_iterType!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_ITER, _iterType);
			if(_taxonomyName!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_TAXONOMY, _taxonomyName);
			if(_taxonomyValue!=null)
				processBeanList.setAttribute(Constants.LBL_SELECTED_TAXONOMY_VALUE, _taxonomyValue);
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredAction::saveParametersForPagination: ", sbe);
		}
	}
	
	protected List getInstanceIterInput(SourceBean serviceRequest){
		List requestParameters = serviceRequest.getContainedAttributes();
		List instanceIterList = new ArrayList();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_ATTRIBUTE ) && !attribute.getValue().equals(""))
				instanceIterList.add(attribute);	
		}
		return instanceIterList;
	}
	
	protected SourceBean createRelevantIterAttributes(String iterType, SourceBean request) {
		SourceBean attributesBean = null;
		try {
			attributesBean = new SourceBean("INSTANCE_ITER");
			List attributeIterList = RelevantDataBO.getAttributes((new Long(iterType)).longValue());
			Iterator attributeIterListIt = attributeIterList.iterator();		
			Attribute attribute = null;
			SourceBean attributeBean = null;
			String idAttribute = null;
			while(attributeIterListIt.hasNext()) {
				attribute = ((AttributeIter)attributeIterListIt.next()).getAttribute();
				attributeBean = new SourceBean("ATTRIBUTE");
				attributeBean.setAttribute(Constants.LBL_ID_ATTRIBUTE, attribute.getIdAttribute());
				attributeBean.setAttribute(Constants.LBL_NAME, attribute.getName());
				idAttribute = Constants.PREFIX_ATTRIBUTE + new Long(attribute.getIdAttribute()).toString();
				if(request.getAttribute(idAttribute) !=null)
					attributeBean.setAttribute(Constants.LBL_VALUE, request.getAttribute(idAttribute));
				else
					attributeBean.setAttribute(Constants.LBL_VALUE, Constants.BLANK);
				attributesBean.setAttribute(attributeBean);					
			} 
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredAction::createRelevantIterAttributes: ", sbe);
		}
		return attributesBean;
	}
	
	protected SourceBean createProcessInstanceBean(List processInstance) 
		throws SourceBeanException {		
		SourceBean itemBean = null;		
		String iter, owner, iterIstanza;
		try {
			itemBean = new SourceBean("ITEM");
			itemBean.setAttribute("id", processInstance.get(0));
			itemBean.setAttribute("idProcesso", processInstance.get(1));
			itemBean.setAttribute("processo", processInstance.get(2));
			iter = (processInstance.get(3)!=null) ? (String)processInstance.get(3) : "";
			itemBean.setAttribute("iter", iter);	
			iterIstanza = (processInstance.get(4)!=null) ? (String)processInstance.get(4) : "";
			itemBean.setAttribute("iterIstanza", decodInstanceIter(iterIstanza));	
			owner = (processInstance.get(5)!=null) ? (String)processInstance.get(5) : "";
			itemBean.setAttribute("owner", owner);
			
			DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.ITALY);
			String dateTemp = formatter.format(processInstance.get(6));					
			itemBean.setAttribute("dataInizio", dateTemp);				
			dateTemp = (processInstance.get(7)!=null) ? formatter.format(processInstance.get(7)) : "";					
			itemBean.setAttribute("dataFine", dateTemp);
										
			itemBean.setAttribute("stato",processInstance.get(8));
			itemBean.setAttribute("restarted",((String)(processInstance.get(9))).startsWith(CORRELATION_ID_PREFIX) ? "TRUE" : "FALSE");
	
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByParametersAction::createProcessInstanceFilteredBean:: ", sbe);
			throw sbe;
		}
		return itemBean;
	}
	
	protected SourceBean createProcessInstanceBean(InstanceSa processInstance) 
		throws SourceBeanException {		
		SourceBean itemBean = null;		
		String iter, instanceIter, owner;
		try {
			itemBean = new SourceBean("ITEM");
			itemBean.setAttribute("id", processInstance.getIdInstanceSa());
			itemBean.setAttribute("idProcesso", processInstance.getServiceAssembly().getIdAssembly());
			itemBean.setAttribute("processo", processInstance.getServiceAssembly().getName());
			iter = "";
			if (processInstance.getInstanceIter() != null)
				iter = (processInstance.getInstanceIter().getIterType().getName()!=null) ? (String)processInstance.getInstanceIter().getIterType().getName() : "";
			itemBean.setAttribute("iter", iter);
			
			instanceIter = (processInstance.getInstanceIter()!=null) ? (String)processInstance.getInstanceIter().getUniqueIterInstanceKey() : "";
			itemBean.setAttribute("iterIstanza", decodInstanceIter(instanceIter));	
			owner = (processInstance.getServiceAssembly().getUddiOrg()!=null) ? (String)processInstance.getServiceAssembly().getUddiOrg() : "";
			itemBean.setAttribute("owner", owner);
			
			DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.ITALY);
			String dateTemp = formatter.format(processInstance.getStartdate());					
			itemBean.setAttribute("dataInizio", dateTemp);				
			dateTemp = (processInstance.getEnddate()!=null) ? formatter.format(processInstance.getEnddate()) : "";					
			itemBean.setAttribute("dataFine", dateTemp);
										
			itemBean.setAttribute("stato",processInstance.getState());
			itemBean.setAttribute("restarted",processInstance.getIdCorrelation().startsWith(CORRELATION_ID_PREFIX) ? "TRUE" : "FALSE");
	
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByParametersAction::createProcessInstanceFilteredBean:: ", sbe);
			throw sbe;
		}
		return itemBean;
		}
	
	private String decodInstanceIter(String uniqueIterInstanceKey) {
		char open = '[';
		char closed = ']';
		char sep = ':';
		char blank = ' ';
		int indexOpen = 0;
		int indexSep = 0;
		int indexClosed = 0;
		String idAttribute;
		String decodInstanceIter = "";
		
		indexOpen = uniqueIterInstanceKey.indexOf(open);
		while(indexOpen != -1) {
			indexSep = uniqueIterInstanceKey.indexOf(sep);
			idAttribute = uniqueIterInstanceKey.substring(indexOpen+1, indexSep);
			decodInstanceIter += ProcessInstanceBO.getAttributeName(Long.parseLong(idAttribute)) + sep;
			indexClosed = uniqueIterInstanceKey.indexOf(closed);
			decodInstanceIter += uniqueIterInstanceKey.substring(indexSep+1, indexClosed) + blank;
			uniqueIterInstanceKey = uniqueIterInstanceKey.substring(indexClosed+1);
			indexOpen = uniqueIterInstanceKey.indexOf(open);
		}	
		
		return decodInstanceIter;
	}

}
