/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;

public class ProcessInstanceListFilteredByParametersAction extends ProcessInstanceListFilteredAction {
	private int _page;
	private String _processo = null;
	private String _version;
	private String _stato = null;
	private String _dataInizio= null;
	private String _dataFine = null;
	private String _owner = null;
	private String _ordinamento = null;
	private String _tipoOrdinamento = null;
	private String _idIterType = null;
	private String _restart = null;
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		_page = getRequestPage(serviceRequest, sessionContainer, Constants.PAGE_INSTANCES);
		
		//get Parameters from request
		loadParametersFromRequest(serviceRequest);
		// recupero (se ci sono) parametri legati all'iter e all'istanza
		List instanceIterInputParameters = getInstanceIterInput(serviceRequest);
		//recupero (se ci sono) attributi rilevanti di processo 
		List relevantAttributeProcessList = getAttributeProcessInput(serviceRequest);
		
		SourceBean processBean = new SourceBean("LIST");	
		//save info for pagination
		saveParametersForPagination(processBean);
		//save info restart
		if(_restart != null)
			serviceResponse.setAttribute(Constants.LBL_RESTART, _restart);
		//genero bean contente attributi di istanza di iter
		if(_idIterType!=null && !_idIterType.equals("-1")) {
			SourceBean attributesList = createRelevantIterAttributes(_idIterType, serviceRequest);
			serviceResponse.setAttribute(attributesList);
		}
		//call ProcessInstanceBO
		List processInstanceList = null;
		Query query= null;
		QueryHibernatePage hibernatePage = null;
		long processIdReal;
		if(!_version.equalsIgnoreCase(Constants.BLANK) && Long.parseLong(_processo) != Constants.ALL_PROCESS) {
			try {
				processIdReal = ProcessBO.getProcessIdByVersion(Integer.parseInt(_version), Long.parseLong(_processo));
			} catch(IndexOutOfBoundsException iobe) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByPArametersAction::service: non esiste la versione paer quasto processo");
				processIdReal = Long.parseLong(_processo);
			}
			query = ProcessInstanceBO.getProcessInstanceList(processIdReal, (new Long(_idIterType)).longValue(), _stato, _dataInizio, _dataFine, _owner, _ordinamento, _tipoOrdinamento, instanceIterInputParameters, relevantAttributeProcessList);				
		} else
			query = ProcessInstanceBO.getProcessInstanceList(Long.parseLong(_processo), (new Long(_idIterType)).longValue(), _stato, _dataInizio, _dataFine, _owner, _ordinamento, _tipoOrdinamento, instanceIterInputParameters, relevantAttributeProcessList);
		
		//pagination
		hibernatePage = new QueryHibernatePage(query, _page, Constants.sizeProcessInstanceListFiltered);
//		if (_page > hibernatePage.getLastPageNumber()){
//			hibernatePage.setPage(--_page);
//			sessionContainer.setAttribute(Constants.PAGE, _page);
//		} else if(_page == Constants.LAST_PAGE_VALUE){
//			int lastPage = hibernatePage.getLastPageNumber();
//			hibernatePage.setPage(lastPage);
//			sessionContainer.setAttribute(Constants.PAGE, lastPage);
//		}
		processInstanceList = hibernatePage.getList();
		
		if (processInstanceList.size()==0) {
			serviceResponse.setAttribute("showProcessSearch", "1");
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10000", "properties.console.messages");
			getErrorHandler().addError(warning);
		}
		
		//create response
		Iterator processIt = processInstanceList.iterator();
		List processInstance = null;
		while (processIt.hasNext()) {
			processInstance = (List)processIt.next();
			processBean.setAttribute(createProcessInstanceBean(processInstance));					
		} 				
		serviceResponse.setAttribute(processBean);
	}
	
	private void loadParametersFromRequest(SourceBean request) {
		_processo = (String)request.getAttribute(Constants.LBL_PROCESS_ID);
		_version = (String)request.getAttribute(Constants.LBL_VERSION);
		_stato = (String)request.getAttribute(Constants.LBL_STATO);
		_dataInizio = (String)request.getAttribute(Constants.LBL_DATA_INIZIO);
		_dataFine = (String) request.getAttribute(Constants.LBL_DATA_FINE);
		_owner = (String)request.getAttribute(Constants.LBL_OWNER);
		_ordinamento = (String)request.getAttribute(Constants.LBL_ORDINAMENTO);
		_tipoOrdinamento = (String)request.getAttribute(Constants.LBL_TIPO_ORDINAMENTO);
		_idIterType = (String)request.getAttribute(Constants.LBL_SELECTED_ITER);
		_restart = (String)request.getAttribute(Constants.LBL_RESTART);
	}
	
	protected List getAttributeProcessInput(SourceBean serviceRequest){
		List requestParameters = serviceRequest.getContainedAttributes();
		List relevantAttributeProcessList = new ArrayList();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_PROCESS_ATTRIBUTE ) && !attribute.getValue().equals(""))
				relevantAttributeProcessList.add(attribute);	
		}
		return relevantAttributeProcessList;
	}
	
	private void saveParametersForPagination(SourceBean processBean) throws SourceBeanException {
		try {
			if(_processo != null)
				processBean.setAttribute(Constants.LBL_PROCESS_ID, _processo);
			if(_version != null)
				processBean.setAttribute(Constants.LBL_VERSION, _version);
			if(_stato != null)
				processBean.setAttribute(Constants.LBL_STATO, _stato);
			if(_dataInizio != null)
				processBean.setAttribute(Constants.LBL_DATA_INIZIO, _dataInizio);
			if(_dataFine != null)
				processBean.setAttribute(Constants.LBL_DATA_FINE, _dataFine);
			if(_owner != null)
				processBean.setAttribute(Constants.LBL_OWNER, _owner);
			if(_ordinamento != null)
				processBean.setAttribute(Constants.LBL_ORDINAMENTO, _ordinamento);
			if(_tipoOrdinamento != null)
				processBean.setAttribute(Constants.LBL_TIPO_ORDINAMENTO, _tipoOrdinamento);
			if(_idIterType!=null)
				processBean.setAttribute(Constants.LBL_SELECTED_ITER, _idIterType);
			if(_restart!=null)
				processBean.setAttribute(Constants.LBL_RESTART, _restart);
		} catch (SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByPArametersAction::saveParametersForPagination: ", sbe);
		}
	}
	
}
