/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.action;

import java.util.Iterator;
import java.util.List;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.process.bo.RelevantDataBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.tracing.TracerSingleton;

public class RelevantDataProcessAndVersionAction extends AbstractAction {
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
		throws Exception {
		String processId = (String)serviceRequest.getAttribute(Constants.LBL_PROCESS_ID);
		if(processId!=null) {
			//version
			serviceResponse.setAttribute(Constants.LBL_VERSION, ProcessBO.getProcessDetail(Long.parseLong(processId)).getSaVersion());
			//process attributes
			SourceBean attributesBean = createProcessAttributeBean(Long.parseLong(processId));
			serviceResponse.setAttribute(attributesBean);
		}
	}
	
	protected SourceBean createProcessAttributeBean(long processId) {
		SourceBean attributesBean = null;
		try {
			attributesBean = new SourceBean("PROCESS_ATTRIBUTES");
			List attributesList = RelevantDataBO.getAttributeRelevant(processId);
			Iterator attributesListIt = attributesList.iterator();		
			List attribute = null;
			SourceBean attributeBean = null;
			String catalog = null;
			while(attributesListIt.hasNext()) {
				attribute = (List)attributesListIt.next();
				attributeBean = new SourceBean("PROCESS_ATTRIBUTE");
				attributeBean.setAttribute(Constants.LBL_ID_ATTRIBUTE, attribute.get(0));
				attributeBean.setAttribute(Constants.LBL_NAME, attribute.get(1));
				catalog = (attribute.get(2) != null)
							? (String)attribute.get(2)
							: "";
				attributeBean.setAttribute(Constants.LBL_CATALOG, catalog);
				attributeBean.setAttribute(Constants.LBL_VALUE, Constants.BLANK);
				attributesBean.setAttribute(attributeBean);					
			}
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RelevantDataProcessAndVersionAction::service():: ", sbe);
		}
		return attributesBean;
	}
}
