/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.AbstractPage;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;

public class MessagesListModule extends MessagesListFilterModule {
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		_page = getRequestPage(serviceRequest, sessionContainer);
		//get Parameters from request
		SourceBean paginationInfo = (SourceBean)serviceRequest.getAttribute("PAGINATION_INFO");
		loadParametersFromRequest(paginationInfo);
		//recupero (se ci sono) attributi rilevanti di processo 
		List relevantAttributeProcessList = getAttributeProcessInput(paginationInfo);
		
		//call ProcessInstanceBO
		List messagesList = null;
		Query query = null;
		AbstractPage hibernatePage = null;
		query = ProcessInstanceBO.getMessagesList(Long.parseLong(_idEndPoint), _stato, _dataInizio, _dataFine, _tipoOrdinamento, relevantAttributeProcessList);
		//pagination
		hibernatePage = new QueryHibernatePage(query, _page, Constants.sizeMessagesList);
		messagesList = hibernatePage.getList();
		
		serviceResponse.setAttribute("LIST", messagesList);
		
		if (messagesList.size()==0) {
			serviceResponse.setAttribute("showMessagesSearch", "1");
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10003", "properties.console.messages");
			getErrorHandler().addError(warning);
		}
	}
	
	private int getRequestPage(SourceBean request, SessionContainer session) {
		String pageFromRequest = (String)request.getAttribute(Constants.PAGE_CMD);
		int page = 0;
		if(pageFromRequest == null || pageFromRequest.equalsIgnoreCase(Constants.FIRST_PAGE))
			page = 0;
		else if(pageFromRequest.equalsIgnoreCase(Constants.PREV_PAGE)) {
			page = ((Integer)session.getAttribute(Constants.PAGE_MESSAGES)).intValue() - 1;
			page = (page < 0) ? 0 : page;
		} 
		else if(pageFromRequest.equalsIgnoreCase(Constants.NEXT_PAGE)) 
			page = ((Integer)session.getAttribute(Constants.PAGE_MESSAGES)).intValue() + 1;
		else if(pageFromRequest.equalsIgnoreCase(Constants.LAST_PAGE)) 
			page = Constants.LAST_PAGE_VALUE;
		else if(pageFromRequest.equalsIgnoreCase(Constants.CURRENT_PAGE))
			page = ((Integer)session.getAttribute(Constants.PAGE_MESSAGES)).intValue();
		
		session.setAttribute(Constants.PAGE_MESSAGES, page);
		return page;
	}

	private List getAttributeProcessInput(SourceBean serviceRequest){
		List requestParameters = serviceRequest.getContainedAttributes();
		List relevantAttributeProcessList = new ArrayList();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_PROCESS_ATTRIBUTE ) && !attribute.getValue().equals(""))
				relevantAttributeProcessList.add(attribute);	
		}
		return relevantAttributeProcessList;
	}
}