/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.hibernate.pagination.QueryHibernatePage;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;

public class ProcessInstanceListFilteredByParametersModule extends ProcessInstanceListFilterModule {
	public static final String CORRELATION_ID_PREFIX = "RESTARTED_ID:";
	
	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		SessionContainer sessionContainer = getRequestContainer().getSessionContainer();
		_page = getRequestPage(serviceRequest, sessionContainer);
		//get Parameters from request
		SourceBean paginationInfo = (SourceBean)serviceRequest.getAttribute("PAGINATION_INFO");
		loadParametersFromRequest(paginationInfo);
		// recupero (se ci sono) parametri legati all'iter e all'istanza
		List instanceIterInputParameters = getInstanceIterInput(paginationInfo);
		//recupero (se ci sono) attributi rilevanti di processo 
		List relevantAttributeProcessList = getAttributeProcessInput(paginationInfo);
		
		SourceBean processBean = new SourceBean("LIST");	

		//call ProcessInstanceBO
		List processInstanceList = null;
		Query query= null;
		QueryHibernatePage hibernatePage = null;
		long processIdReal;
		if(!_version.equalsIgnoreCase(Constants.BLANK) && Long.parseLong(_processo) != Constants.ALL_PROCESS) {
			try {
				processIdReal = ProcessBO.getProcessIdByVersion(Integer.parseInt(_version), Long.parseLong(_processo));
			} catch(IndexOutOfBoundsException iobe) {
				TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByPArametersAction::service: non esiste la versione paer quasto processo");
				processIdReal = Long.parseLong(_processo);
			}
			query = ProcessInstanceBO.getProcessInstanceList(processIdReal, (new Long(_idIterType)).longValue(), _stato, _dataInizio, _dataFine, _owner, _ordinamento, _tipoOrdinamento, instanceIterInputParameters, relevantAttributeProcessList);				
		} else
			query = ProcessInstanceBO.getProcessInstanceList(Long.parseLong(_processo), (new Long(_idIterType)).longValue(), _stato, _dataInizio, _dataFine, _owner, _ordinamento, _tipoOrdinamento, instanceIterInputParameters, relevantAttributeProcessList);
		
		//pagination
		hibernatePage = new QueryHibernatePage(query, _page, Constants.sizeProcessInstanceListFiltered);
		processInstanceList = hibernatePage.getList();
		
		if (processInstanceList.size()==0 && _page==0) {
			serviceResponse.setAttribute("showProcessSearch", "1");
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10000", "properties.console.messages");
			getErrorHandler().addError(warning);
		}
		
		//create response
		Iterator processIt = processInstanceList.iterator();
		List processInstance = null;
		while (processIt.hasNext()) {
			processInstance = (List)processIt.next();
			processBean.setAttribute(createProcessInstanceBean(processInstance));					
		} 				
		serviceResponse.setAttribute(processBean);
	}
	private int getRequestPage(SourceBean request, SessionContainer session) {
		String pageFromRequest = (String)request.getAttribute(Constants.PAGE_CMD);
		int page = 0;
		if(pageFromRequest == null || pageFromRequest.equalsIgnoreCase(Constants.FIRST_PAGE))
			page = 0;
		else if(pageFromRequest.equalsIgnoreCase(Constants.PREV_PAGE)) {
			page = ((Integer)session.getAttribute(Constants.PAGE_INSTANCES)).intValue() - 1;
			page = (page < 0) ? 0 : page;
		} 
		else if(pageFromRequest.equalsIgnoreCase(Constants.NEXT_PAGE)) 
			page = ((Integer)session.getAttribute(Constants.PAGE_INSTANCES)).intValue() + 1;
		else if(pageFromRequest.equalsIgnoreCase(Constants.LAST_PAGE)) 
			page = Constants.LAST_PAGE_VALUE;
		else if(pageFromRequest.equalsIgnoreCase(Constants.CURRENT_PAGE))
			page = ((Integer)session.getAttribute(Constants.PAGE_INSTANCES)).intValue();
		
		session.setAttribute(Constants.PAGE_INSTANCES, page);
		return page;
	}

	protected List getAttributeProcessInput(SourceBean serviceRequest){
		List requestParameters = serviceRequest.getContainedAttributes();
		List relevantAttributeProcessList = new ArrayList();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_PROCESS_ATTRIBUTE ) && !attribute.getValue().equals(""))
				relevantAttributeProcessList.add(attribute);	
		}
		return relevantAttributeProcessList;
	}
	
	protected List getInstanceIterInput(SourceBean serviceRequest){
		List requestParameters = serviceRequest.getContainedAttributes();
		List instanceIterList = new ArrayList();
		Iterator it = requestParameters.iterator();
		SourceBeanAttribute attribute = null;
		while(it.hasNext()) {
			attribute = (SourceBeanAttribute)it.next();
			if (attribute.getKey().contains(Constants.PREFIX_ATTRIBUTE ) && !attribute.getValue().equals(""))
				instanceIterList.add(attribute);	
		}
		return instanceIterList;
	}
	

	protected SourceBean createProcessInstanceBean(List processInstance) 
	throws SourceBeanException {		
		SourceBean itemBean = null;		
		String iter, owner, iterIstanza;
		try {
			itemBean = new SourceBean("ITEM");
			itemBean.setAttribute("id", processInstance.get(0));
			itemBean.setAttribute("idProcesso", processInstance.get(1));
			itemBean.setAttribute("processo", processInstance.get(2));
			iter = (processInstance.get(3)!=null) ? (String)processInstance.get(3) : "";
			itemBean.setAttribute("iter", iter);	
			iterIstanza = (processInstance.get(4)!=null) ? (String)processInstance.get(4) : "";
			itemBean.setAttribute("iterIstanza", decodInstanceIter(iterIstanza));	
			owner = (processInstance.get(5)!=null) ? (String)processInstance.get(5) : "";
			itemBean.setAttribute("owner", owner);
			
			DateFormat formatter = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, Locale.ITALY);
			String dateTemp = formatter.format(processInstance.get(6));					
			itemBean.setAttribute("dataInizio", dateTemp);				
			dateTemp = (processInstance.get(7)!=null) ? formatter.format(processInstance.get(7)) : "";					
			itemBean.setAttribute("dataFine", dateTemp);
										
			itemBean.setAttribute("stato",processInstance.get(8));
			itemBean.setAttribute("restarted",((String)(processInstance.get(9))).startsWith(CORRELATION_ID_PREFIX) ? "TRUE" : "FALSE");
	
		} catch(SourceBeanException sbe) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceListFilteredByParametersAction::createProcessInstanceFilteredBean:: ", sbe);
			throw sbe;
		}
		return itemBean;
	}
	
	private String decodInstanceIter(String uniqueIterInstanceKey) {
		char open = '[';
		char closed = ']';
		char sep = ':';
		char blank = ' ';
		int indexOpen = 0;
		int indexSep = 0;
		int indexClosed = 0;
		String idAttribute;
		String decodInstanceIter = "";
		
		indexOpen = uniqueIterInstanceKey.indexOf(open);
		while(indexOpen != -1) {
			indexSep = uniqueIterInstanceKey.indexOf(sep);
			idAttribute = uniqueIterInstanceKey.substring(indexOpen+1, indexSep);
			decodInstanceIter += ProcessInstanceBO.getAttributeName(Long.parseLong(idAttribute)) + sep;
			indexClosed = uniqueIterInstanceKey.indexOf(closed);
			decodInstanceIter += uniqueIterInstanceKey.substring(indexSep+1, indexClosed) + blank;
			uniqueIterInstanceKey = uniqueIterInstanceKey.substring(indexClosed+1);
			indexOpen = uniqueIterInstanceKey.indexOf(open);
		}	
		
		return decodInstanceIter;
	}
}