/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.audit.module;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.List;

import org.hibernate.Query;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.util.Constants;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.action.AbstractAction;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spagosoa.smx.components.restart.client.RestartResult;
import it.eng.spagosoa.smx.components.restart.client.RestartServiceProxy;

public class RestartAllInstancesModule extends ProcessInstanceListFilteredByParametersModule {

	public void service(SourceBean serviceRequest, SourceBean serviceResponse)
			throws Exception {
		//get Parameters from request
		SourceBean paginationInfo = (SourceBean)serviceRequest.getAttribute("PAGINATION_INFO");
		loadParametersFromRequest(paginationInfo);
		
		// recupero (se ci sono) parametri legati all'iter e all'istanza
		List instanceIterInputParameters = getInstanceIterInput(paginationInfo);
		//recupero (se ci sono) attributi rilevanti di processo 
		List relevantAttributeProcessList = getAttributeProcessInput(paginationInfo);

		//effettuo query e preparo input per WebService
		List processInstanceIdList = ProcessInstanceBO.getProcessInstanceIdList(Long.parseLong(_processo), (new Long(_idIterType)).longValue(), _stato, _dataInizio, _dataFine, _owner, _ordinamento, _tipoOrdinamento, instanceIterInputParameters, relevantAttributeProcessList);
		int idRestartSize = processInstanceIdList.size();
		long[] idRestartList = new long[idRestartSize];
		int indexIdRestart;
		for (indexIdRestart=0; indexIdRestart<idRestartSize; indexIdRestart++)
			idRestartList[indexIdRestart] = (Long)processInstanceIdList.get(indexIdRestart);  
			
		//call method to restart all Instances
		RestartServiceProxy proxy = new RestartServiceProxy();
		String locationRestartWS = (String)ConfigSingleton.getInstance().getAttribute("CONSOLE.locationRestartWS");
		proxy.setEndpoint(locationRestartWS);
		RestartResult[] restartResult = null;
		try {
			// Restart process
			restartResult = proxy.restartMultipleInstances(idRestartList);
		} catch(RemoteException re) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "RestartAllInstancesModule::service:: ", re);
			EMFUserError warning = new EMFUserError(EMFErrorSeverity.INFORMATION, "10001", "properties.console.messages");
			getErrorHandler().addError(warning);
		}
		
		serviceResponse.setAttribute("idOriginal",idRestartList);
		serviceResponse.setAttribute("idRestarted",restartResult);
		
	}

}
