/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.hibernate.pagination;

import java.util.List;

import it.eng.console.util.Constants;
import it.eng.spago.tracing.TracerSingleton;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;

public class CriteriaHibernatePage extends AbstractPage{

	private Criteria criteria;

	/**
	 * Construct a new Page. Page numbers are zero-based, so the first page is
	 * page 0.
	 * 
	 * @param query
	 *            the Hibernate Query
	 * @param page
	 *            the page number (zero-based)
	 * @param pageSize
	 *            the number of results to display on the page
	 */

	public CriteriaHibernatePage(Criteria criteria, int page, int pageSize) throws HibernateException {
		super(page, pageSize);
		try {
			this.criteria = criteria;
			//scrollableResults = criteria.scroll();
		} catch (HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "HibernatePage::HibernatePage: fallita impaginazione", he);
            throw he;
		}
	}
	
	protected List getResults() {
		/*
		 * We set the max results to one more than the specfied pageSize to
		 * determine if any more results exist (i.e. if there is a next page
		 * to display). The result set is trimmed down to just the pageSize
		 * before being displayed later (in getList()).
		 */
		try {
			results = criteria.setFirstResult(page * pageSize).setMaxResults(
					pageSize + 1).list();
		} catch (HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "HibernatePage::HibernatePage: fallita impaginazione", he);
            throw he;
		}
		return results;
	}
}
