/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.parser;

import it.eng.console.audit.bo.ProcessInstanceBO;
import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.base.ServiceAssemblyStateConstants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.tracing.TracerSingleton;

import java.util.Iterator;
import java.util.List;

public class SvgParser {
	long _processId;
	long _processInstanceId;
	//_svgBean can contains processSvg or processInstanceSvg
	SourceBean _svgBean = null;	
	static String NODE_LABEL = "node";
	static String EDGE_LABEL = "edge";
	
	public SvgParser(long processId) throws SourceBeanException {
		_processId = processId;
		initInputSvgBean();					
	}

	public SvgParser(long processId, long processInstanceId) throws SourceBeanException {
		_processId = processId;
		_processInstanceId = processInstanceId;
		initInputSvgBean();
	}
	
	private void initInputSvgBean() throws SourceBeanException {
		try {
			String roughSvg = ProcessBO.getRoughSvg(_processId);
			_svgBean = SourceBean.fromXMLString(roughSvg, true, false);			
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::initInputSvgBean: ", sbe);
			throw sbe;
		}		
	}

	public String createProcessSvgString() throws SourceBeanException {
		try {			
			String width = (String)_svgBean.getAttribute("width");
			_svgBean.updAttribute("width", getBiggerWidth(width));
			List nodeList = _svgBean.getFilteredSourceBeanAttributeAsList("g.g", "class", NODE_LABEL);
			Iterator nodeIt = nodeList.iterator();
			SourceBean nodeBean = null;
			String endPointId = null;		
			Endpoint endPoint = null;			
			while(nodeIt.hasNext()) {
				nodeBean = (SourceBean)nodeIt.next();
				nodeBean.updAttribute("a.href", "#");
				endPointId = nodeBean.getCharacters("a.text");
				modifyCoordinateNextText((String)nodeBean.getAttribute("a.text.y"));
				nodeBean.updAttribute("id", endPointId);
				nodeBean.setAttribute("onclick", "showDetail(evt)");
				endPoint = ProcessBO.getEndPointDetail((new Long(endPointId)).longValue());				
				nodeBean.setCharacters("a.text", endPoint.getName());
				nodeBean.updAttribute("a.text.style", "font-size:6.00pt;");				
			}			
			_svgBean.setAttribute(getCDATABean());			
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::createProcessSvgString: ", sbe);
			throw sbe;
		}
		return _svgBean.toString();
	}
	
	private void modifyCoordinateNextText(String y) throws SourceBeanException {			
		List textBeanList = _svgBean.getFilteredSourceBeanAttributeAsList("g.g", "a.text.y", y);		
		Iterator it = textBeanList.iterator();
		SourceBean textBean = null;
		try {
			if(textBeanList.size()>1) {
				int startY = Integer.parseInt(y) - 6;
				int distance = 11;
				while(it.hasNext()) {
					textBean = (SourceBean)it.next();
					textBean.updAttribute("a.text.y", Integer.toString(startY));
					startY += distance; 					
				}
			}
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::modifyCoordinateNextText: ", sbe);
			throw sbe;
		}
	}
	
	private SourceBean getCDATABean() throws SourceBeanException{
		SourceBean scriptBean = null;
		try {
			scriptBean = new SourceBean("script", true, false);
			scriptBean.setAttribute("type", "text/javascript");
			String functionString = "function showDetail(evt){\n"
								  + "var elem = evt.target.parentNode.parentNode;\n"
								  + "var endPointId = elem.getAttribute('id');\n"
								  + "parent.showDetail(endPointId);\n"
								  + "}";
			scriptBean.setCharacters(functionString);
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::getCDATABean: ", sbe);
			throw sbe;
		}
		return scriptBean;
	}
	
	private String getBiggerWidth(String width) {
		if(width.indexOf("pt") != -1) {
			String widthTemp = width.substring(0, width.indexOf("pt")); 
			int newWidth = Integer.parseInt(widthTemp) + 70; 
			return Integer.toString(newWidth) + "pt";
		} else {
			String widthTemp = width.substring(0, width.indexOf("in")); 
			double newWidth = Double.parseDouble(widthTemp) + 2; 
			return Double.toString(newWidth) + "in";
		}
	}

	public String createProcessInstanceSvgString() throws SourceBeanException {
		createProcessSvgString();
		List nodeList = _svgBean.getFilteredSourceBeanAttributeAsList("g.g", "class", NODE_LABEL);
		Iterator nodeListIterator = nodeList.iterator();
		SourceBean nodeBean = null;
		long endPointId;
		int stateInstanceService;
		while(nodeListIterator.hasNext()) {
			nodeBean = (SourceBean)nodeListIterator.next();
			endPointId = Long.parseLong((String)nodeBean.getAttribute("id"));	
			stateInstanceService = ProcessInstanceBO.getExecutionStateInstanceService(endPointId, _processInstanceId);
			changeNodeState(stateInstanceService, nodeBean);				
		}		
		return _svgBean.toString();
	}

	private void changeNodeState(int state, SourceBean node) throws SourceBeanException{
		String newColor = null;
		switch (state) {
		case ServiceAssemblyStateConstants.SA_DONE:
			newColor = "#99FF99";//"#33FF99"
			break;
		case ServiceAssemblyStateConstants.SA_ACTIVE:
			newColor = "#FFFF99";
			break;			
		case ServiceAssemblyStateConstants.SA_FAULTED:
			newColor = "red";
			break;
		}
		SourceBean polygon = (SourceBean)node.getAttribute("a.polygon");
		changeColor("gray", newColor, polygon);
		List nodeList = node.getAttributeAsList("a.polyline");
		if (nodeList != null)
			for (int index=0; index<nodeList.size(); index++) 
				changeColor("gray", newColor, (SourceBean)nodeList.get(index));		
	}
	
	private void changeColor(String oldColor, String newColor, SourceBean node) 
		throws SourceBeanException {
		String style = (String)node.getAttribute("style");		
		style = style.replace("none", newColor);
		style = style.concat("fill-opacity:0.3;");
		try {
			node.updAttribute("style", style);
		} catch(SourceBeanException sbe) {		
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "SvgParser::changeColor: ", sbe);
			throw sbe;
		}
	}
}
