/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.process.bo;

import it.eng.console.hibernate.util.HibernateSessionFactory;
import it.eng.console.util.Constants;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.IterType;
import it.eng.spagosoa.metadb.model.SaSvg;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.serviceregistry.IServiceRegistry;
import it.eng.serviceregistry.JAXRServiceRegistryImpl;
import it.eng.serviceregistry.ServiceRegistryException;
import it.eng.spago.tracing.TracerSingleton;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.Service;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Restrictions;

public class ProcessBO {
	public static Criteria getProcessList() throws HibernateException{
		Criteria criteria = null;
		try {
			DetachedCriteria lastVersion = DetachedCriteria.forClass(ServiceAssembly.class,"sa2")
											.setProjection(Property.forName("saVersion").max())
											.add( Property.forName("sa2.name").eqProperty("sa.name") );
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(ServiceAssembly.class, "sa")		
								.addOrder(Order.asc("name"))
								.add(Property.forName("saVersion").eq(lastVersion)); 
										
			
			return criteria;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", he);
			throw he;
		}
	}

	public static Criteria getProcessList(long idIterType) throws HibernateException{
		Criteria criteria = null;
		try {
			criteria = getProcessList();

			if(idIterType == Constants.NO_ITER) 
				criteria.add(Restrictions.isEmpty("iterTypeServiceAssemblies"));
			else if(idIterType != Constants.ALL_ITER)
				criteria = criteria.createCriteria("iterTypeServiceAssemblies", Criteria.INNER_JOIN)
							.add(Expression.eq("iterType.idIterType", idIterType));		
			
			return criteria;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList(long idIterType): ", he);
			throw he;
		}
	}	
	
	public static ServiceAssembly getProcessDetail(long processId) throws HibernateException{
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			ServiceAssembly serviceAssembly = (ServiceAssembly)session.get(ServiceAssembly.class, processId);	
			return serviceAssembly;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessDetail: ", he);
			throw he;
		}
	}
	
	public static Criteria getProcessList(String taxonomyName, String taxonomyValue) throws Exception{			
		IServiceRegistry serviceRegistry = null;
		Criteria criteria = null;
		try {
			serviceRegistry = new JAXRServiceRegistryImpl();
			serviceRegistry.initRegistry();
			List serviceList = serviceRegistry.getServicesClassifiedByTaxonomy(taxonomyName, taxonomyValue);
			int indexService;
			Service service;
			String uddiName, uddiOrganization;
			
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(ServiceAssembly.class);
			
			
			if(serviceList.size()>0) {
				Disjunction orExpressions = Restrictions.disjunction();
				for(indexService=0; indexService<serviceList.size(); indexService++) {
					service = (Service)serviceList.get(indexService);
					uddiName = service.getName().getValue();				
					uddiOrganization = service.getProvidingOrganization().getName().getValue();
					orExpressions.add(Expression.and(Expression.eq("uddiOrg", uddiOrganization), Expression.eq("uddiName", uddiName)));
				}
				criteria.add(orExpressions);
			}
			else
				criteria.add(Expression.eq("idAssembly", "-1"));
			
			criteria.addOrder(Order.asc("name"));
				
		} catch(ServiceRegistryException sre) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", sre);
			throw sre;
		} catch(JAXRException jax) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", jax);
			throw jax;
		} catch(Exception ex) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessList: ", ex);
			throw ex;
		} finally {
			serviceRegistry.finalizeRegistry();			
		}
		return criteria;
	}
	

	public static Endpoint getEndPointDetail(long endPointId) {
		try {
			Endpoint endPoint = null;
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			endPoint = (Endpoint)session.get(Endpoint.class, endPointId);			
			return endPoint;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getEndPointDetail: ", he);
			throw he;
		}
	}	
	public static List getEndPointList(long processId) {
		Criteria criteria = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			criteria = session.createCriteria(Endpoint.class);
			criteria.add(Expression.eq("serviceAssembly.idAssembly", processId));
			criteria.addOrder(Order.asc("name"));
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getEndPointDetail: ", he);
			throw he;
		}
		return criteria.list();
	}	
	
	public static Set getEndPointProperties(long endPointId) {
		try {
			Endpoint endPoint = null;
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			endPoint = (Endpoint)session.get(Endpoint.class, endPointId);			
			return endPoint.getPropertyEndpoints();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getEndPointProperties: ", he);
			throw he;
		}
	}	
	
	public static String getRoughSvg(long processId) {
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			SaSvg svg = (SaSvg)session.get(SaSvg.class, processId);
			return svg.getSvgDiag();
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getRoughSvg: ", he);
			throw he;
		}
	}

	public static List getProcessListOrderByName() throws HibernateException{		
		try {
			Criteria criteria = getProcessList();		
			List processList = criteria.list();
			return processList;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getProcessListOrderByName: ", he);
			throw he;
		}
	}
	
	public static List getAllOrganizations(){
		IServiceRegistry serviceRegistry = null;
		List orgList = new ArrayList();
		try {
			serviceRegistry = new JAXRServiceRegistryImpl();
			serviceRegistry.initRegistry();
			orgList = serviceRegistry.getAllOrganizations();
		} catch(ServiceRegistryException sre) {
			TracerSingleton.log(Constants.project, TracerSingleton.INFORMATION, "ProcessBO::getAllOraganizations: ", sre);
		} finally {
			serviceRegistry.finalizeRegistry();	
			return orgList;
		}	
	}
	
	public static List getIterList()	{	
		List iterList = null;
		try {
			Session session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			Criteria criteria = session.createCriteria(IterType.class);		
			criteria.addOrder(Order.asc("name"));
			iterList = criteria.list();
			return iterList;
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessBO::getIterList: ", he);
			throw he;
		}
	}

	public static long getProcessIdByVersion(int version, long processIdOfOtherVersiono) {
		Session session = null;
		Long processId;
		try {
			session = HibernateSessionFactory.getSessionFactory().getCurrentSession();
			String hqlString = "SELECT serviceAssembly.idAssembly " 	 
							+ " FROM ServiceAssembly serviceAssembly "								
							+ " WHERE serviceAssembly.saVersion = :version"
							+ " AND serviceAssembly.name = (SELECT saSubselect.name " 
							+								" FROM ServiceAssembly saSubselect"
							+ 								" WHERE saSubselect.idAssembly = :idAssemblyInput)";
			Query query = session.createQuery(hqlString);
			query.setInteger("version", version);
			query.setLong("idAssemblyInput", processIdOfOtherVersiono);
			
			processId = (Long)query.list().get(0);
		} catch(HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "ProcessInstanceBO::getProcessId:: ", he);
			throw he;
		} 		
		return processId;
	}
}
