/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.tag;

import java.util.Iterator;
import java.util.List;

import it.eng.console.process.bo.ProcessBO;
import it.eng.console.util.Constants;
import it.eng.console.util.HTMLContentGenerator;
import it.eng.spagosoa.metadb.model.IterType;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

public class IterTypeComboTag extends TagSupport {	
	protected String _actionName = null;
	protected String _jsFunctionName = null;
	protected String _idSelect = null;
	protected boolean _showAttributes = false;
	protected StringBuffer _outputHtml = null;
	protected SourceBean _serviceResponse = null;
	protected  long _idIterType = Constants.NOT_SELECTED;
	
    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "IterTypeComboTag::doStartTag:: invocato");
        if(pageContext == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
                "IterTypeComboTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } 
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } catch(Exception ex) {
            TracerSingleton.log(
            		Constants.project, TracerSingleton.CRITICAL, "IterTypeComboTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        if(httpRequest == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "IterTypeComboTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        } 
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if(responseContainer == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "IterTypeComboTag::doStartTag:: httpRequest nullo");
            throw new JspException("responseContainer nullo");
        }        
        
    	_serviceResponse = responseContainer.getServiceResponse();
    	
    	generateForm();
        try {
            pageContext.getOut().print(_outputHtml.toString());
        } catch(Exception ex) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "IterTypeComboTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        }
        return SKIP_BODY;
    } 
    
    private void generateForm() {
    	_outputHtml = new StringBuffer();
    	if(_showAttributes) {
	    	_outputHtml.append("<table>\n");
	    	_outputHtml.append("<tr>\n");
    	}
    	_outputHtml.append("<td><label for=\"iterType\">" + getLabel("iterType") + ":</label></td>\n");
    	_outputHtml.append("<td>");
    	generateIterTypeCombo();
		_outputHtml.append("</td>\n");
		if(_showAttributes) {
			_outputHtml.append("</tr>\n");
			_outputHtml.append("</table>\n");
			_outputHtml.append("<div id=\"inputFieldsRelevantDataAttributes\">\n");
			_outputHtml.append("<table>\n");
			generateInputFieldsRelevantDataAttributes();
			_outputHtml.append("</table>\n");
			_outputHtml.append("</div>\n"); 
		}
    }
    
    private void generateIterTypeCombo() {
    	_outputHtml.append("<select name=\"" + _idSelect + "\" id=\"" + _idSelect + "\"");
        if(_actionName != null && _jsFunctionName != null)
        	_outputHtml.append(" onchange=\"" + _jsFunctionName + "('" + Constants.contextRoot + "/" + _actionName + ".action?" + Constants.LBL_SELECTED_ITER + "='+this.value)\">\n");
        else 
        	_outputHtml.append(">\n");
        
        _outputHtml.append("<option value=\"" + Constants.ALL_ITER + "\">" + getLabel("allIter") + "</option>");
        _outputHtml.append("<option value=\"" + Constants.NO_ITER + "\">" + getLabel("noIter") + "</option>");
         				
		List iterList = ProcessBO.getIterList();	
		Iterator iterListIt = iterList.iterator();
		IterType iter;	
		String idSelectedIterType = (String)_serviceResponse.getAttribute("LIST.idSelectedIterType");
		if(idSelectedIterType!=null)  
			_idIterType = new Long(idSelectedIterType).longValue();		
		while(iterListIt.hasNext()) {
			iter = (IterType)iterListIt.next();
			if(idSelectedIterType!=null && _idIterType == iter.getIdIterType())				
				_outputHtml.append("<option value=\"" + iter.getIdIterType()+ "\" selected>" + iter.getName() + "</option>");
			else 
				_outputHtml.append("<option value=\"" + iter.getIdIterType()+ "\">" + iter.getName() + "</option>");
		}
		_outputHtml.append("</select>\n");	
    }
    
    private void generateInputFieldsRelevantDataAttributes() {
    	if(_serviceResponse.getAttribute("INSTANCE_ITER")==null)
    		return;
    	
    	List attributesList = ((SourceBean)_serviceResponse.getAttribute("INSTANCE_ITER")).getAttributeAsList("ATTRIBUTE");
    	_outputHtml.append(HTMLContentGenerator.getAttributesIterHTML(attributesList, _idSelect, _actionName));
    }
    
    private String getLabel(String id) {
    	return MessageBundle.getMessage(id, "properties.spagic");
    }
    
	public String getIdSelect() {
		return _idSelect;
	}

	public void setIdSelect(String idSelect) {
		_idSelect = idSelect;
	}

	public String getActionName() {
		return _actionName;
	}

	public void setActionName(String actionName) {
		_actionName = actionName;
	}
	
	public String getJsFunctionName() {
		return _jsFunctionName;
	}

	public void setJsFunctionName(String function) {
		_jsFunctionName = function;
	}

	public boolean isShowAttributes() {
		return _showAttributes;
	}

	public void setShowAttributes(boolean showAttributes) {
		_showAttributes = showAttributes;
	}

	/**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "IterTypeComboTag::doEndTag:: invocato");
        return super.doEndTag();
    }
}
