/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.jmx.tag;

import it.eng.console.util.Constants;
import it.eng.jmx.bo.JmxBO;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.ResponseContainerAccess;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

public class QueueListTag extends TagSupport {
	private String type = null; 
	
    public int doStartTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "QueueListTag::doStartTag:: invocato");
        if(pageContext == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL,
                "QueueListTag::doStartTag:: pageContext nullo");
            throw new JspException("pageContext nullo");
        } 
        HttpServletRequest httpRequest = null;
        try {
            httpRequest = (HttpServletRequest) pageContext.getRequest();
        } catch(Exception ex) {
            TracerSingleton.log(
            		Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        if(httpRequest == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag:: httpRequest nullo");
            throw new JspException("httpRequest nullo");
        }
        ResponseContainer responseContainer =
            ResponseContainerAccess.getResponseContainer(httpRequest);
        if(responseContainer == null) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag:: httpRequest nullo");
            throw new JspException("responseContainer nullo");
        }    
        StringBuffer output = new StringBuffer(); 
        SourceBean serviceResponse = responseContainer.getServiceResponse();
		List queueList = (List)serviceResponse.getAttribute(type);
		if (queueList == null) {
			output.append("<div class='div_detail_form'>\n");
	    	output.append("<span class='portlet-form-input-field'>Il server potrebbe non essere raggiungibile.</span>\n");
	    	output.append("</div>");
		} else {
			output.append("<table style='width:100%;margin-top:1px;height:30%;' >\n");	
			AttributeList attributeValues;
			int indexList, indexAttributes;
			
			//intestazione colonne
			ConfigSingleton config = ConfigSingleton.getInstance();
			String idDefaultServer= (String)config.getAttribute("SERVERS.default");
			String[] label = null;
			if(type.equals(it.eng.jmx.util.Constants.LBL_TOPIC)) 
				label = JmxBO.getTopicAttributes(idDefaultServer);
			else 
				label = JmxBO.getQueueAttributes(idDefaultServer);
			output.append("<tr>\n");
			for(int index = 0; index < label.length; index++)
				output.append("<td class='portlet-section-header' style='vertical-align:middle;text-align:left;'  >" +  label[index] + "&nbsp;</td>\n");
			output.append("</tr>\n");
			
			//contenuto righe tabella
			boolean alternate = true;
			for(indexList=0; indexList<queueList.size(); indexList++) {
				attributeValues = (AttributeList)queueList.get(indexList);
				output.append("<tr class='portlet-font'>\n");
				for (indexAttributes=0; indexAttributes<attributeValues.size(); indexAttributes++) {
					if (alternate) {
						output.append("<td class='portlet-section-alternate'");
					}
					else {
						output.append("<td class='portlet-section-body'");
					}
					output.append("style='vertical-align:middle;text-overflow:ellipsis;text-align:left;' >");
					output.append(((Attribute)attributeValues.get(indexAttributes)).getValue() + "</td>\n"); 
				}
				if (alternate) 
					alternate = false;
				else alternate = true;
				
				
				output.append("<tr>\n");
			}		
			
			output.append("</table>\n");	
		}
		
        try {
            pageContext.getOut().print(output.toString());
        } catch(Exception ex) {
            TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "QueueListTag::doStartTag::", ex);
            throw new JspException(ex.getMessage());
        } 
        return SKIP_BODY;
    }
    
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	/**
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag() throws JspException {
        TracerSingleton.log(Constants.project, TracerSingleton.DEBUG, "QueueListTag::doEndTag:: invocato");
        return super.doEndTag();
    } 

}
